package hemesher;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;

import org.sunflow.math.Point3;
import org.sunflow.math.Vector3;

import processing.core.PApplet;
import sunflowapiapi.P5SunflowAPIAPI;
import wblut.hemesh.*;
import wblut.hemesh.creators.HEC_Box;
import wblut.hemesh.creators.HEC_Cone;
import wblut.hemesh.creators.HEC_Creator;
import wblut.hemesh.creators.HEC_Octahedron;
import wblut.hemesh.creators.HEC_Tetrahedron;
import wblut.hemesh.modifiers.HEM_Extrude;
import wblut.hemesh.modifiers.HEM_Lattice;
import wblut.hemesh.subdividors.HES_CatmullClark;

public class Hemesher extends PApplet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	HE_Selection selection;
	HE_Mesh box;
	HE_Mesh blank;
	
	int sceneWidth = 640;
	int sceneHeight = 480;
	P5SunflowAPIAPI sunflow;
	
	public void setup() {
		size(sceneWidth, sceneHeight, "sunflowapiapi.P5SunflowAPIAPI");
		sunflow = (P5SunflowAPIAPI) g;

		// HEC_Creator creator=new HEC_Cube(this).setEdge(100).setCenter(-100,-100,-100).setZAxis(1,1,1).setZAngle(PI/3).setToModelview(false);
		// setWidthSegments(5)
		// setHeigthSegments(5)
		// setDepthSegments(5)
		// setInnerRadius(50)
		// setOuterRadius(50)
		// setMidRadius(50)
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		// setCenter(),setZAxis(), setZAngle() or setToModelview() hould be called last. 
		// These return the generic HEC_Creator object which has no access 
		// to the specific creator object. This would return an error:
		// creator=new HEC_Cube(this).setCenter(-100,-100,-100).setEdge(100);

		/*
		* Box
		*/

		// creator=new HEC_Box(this).setWidth(100).setHeight(50).setDepth(40).setCenter(100,-100,-100).setToModelview(false);
		// setWidthSegments(5)
		// setHeigthSegments(5)
		// setDepthSegments(5)
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		/*
		* Dodecahedron
		*/

		// creator=new HEC_Dodecahedron(this).setOuterRadius(50).setCenter(-100,100,-100).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();
		// setEdge(50)
		// setInnerRadius(50)
		// setMidRadius(50)

		/*
		* Icosahedron
		*/

		// creator=new HEC_Icosahedron(this).setOuterRadius(50).setCenter(-100,-100,100).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();
		// setEdge(50)
		// setInnerRadius(50)
		// setMidRadius(50)

		/*
		* Tetrahedron
		*/

		//HEC_Creator boxCreator=new HEC_Tetrahedron(this).setOuterRadius(50).setCenter(100,-100,100).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();
		// setEdge(50)
		// setInnerRadius(50)
		// setMidRadius(50)

		/*
		* Octahedron
		*/

		//HEC_Creator boxCreator = new HEC_Octahedron(this).setOuterRadius(50).setCenter(100,100,-100).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();
		// setEdge(50)
		// setInnerRadius(50)
		// setMidRadius(50)

		/*
		* Sphere
		*/

		// creator=new HEC_Sphere(this).setRadius(50).setUFacets(16).setVFacets(8).setCenter(100,100,100).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		/*
		* Cylinder
		*/

		// creator=new HEC_Cylinder(this).setRadius(20).setHeight(100).setFacets(16).setCenter(-100,100,100).setZAxis(-1,1,-1).setToModelview(false);
		//setRadius(20,40)
		//setSteps(5)
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		/*
		* Cone
		*/

		// creator=new HEC_Cone(this).setRadius(40).setHeight(100).setFacets(16).setCenter(-100,-100,200).setZAxis(0,0,1).setToModelview(false);
		//setSteps(5)
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		/*
		* Geodesic sphere
		*/

		// creator=new HEC_Geodesic(this).setRadius(40).setLevel(3).setCenter(-100,100,200).setToModelview(false);
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();

		/*
		* Flat grid
		*/

		// creator=new HEC_Grid(this).setResU(10).setResV(10).setSizeU(100).setSizeV(100).setCenter(0,200,-100).setToModelview(false);
		// setWValues(float[10][10])
		// mesh=new HE_Mesh(creator);
		// mesh.drawFaces();
		
		
		  HEC_Box boxCreator=new HEC_Box(this).setWidth(200).setWidthSegments(10).setHeight(200).setHeightSegments(10).setDepth(200).setDepthSegments(10);
		  boxCreator.setCenter(0,0,0).setAxis(1,1,1,1,1,1);
		  box=new HE_Mesh(boxCreator);
		  HEM_Lattice lattice=new HEM_Lattice().setDepth(5).setWidth(5).setFuse(true).setThresholdAngle(HALF_PI);
		  box.modify(lattice);
		  
			blank = new HE_Mesh(this);
		
		  //HEC_Creator boxCreator=new HEC_Cone(this).setRadius(40).setHeight(100).setFacets(16).setCenter(-100,-100,200).setZAxis(0,0,1).setToModelview(false);
		
		  
		  HE_Face faces[] = box.facesAsArray();
		  boolean once_again = false;
		  int originalLength = faces.length;
		  for(int i=0; i<faces.length; i++){
			  ArrayList<HE_Vertex> vertexes = faces[i].vertices();
			  if(vertexes.size()>3){
				  //box.triangulateFace(faces[i].key(), true);
				  box.triSplitFace(faces[i]);
				  once_again = true;
			  }else{
				  blank.add(faces[i]);
			  }
		  }
		  
		  int once_count = 0;
		  while(once_again){
			  faces = box.facesAsArray();
			  int t_length = originalLength;
			  originalLength = faces.length;
			  once_again = false;
			  for(int i=t_length-1; i<faces.length; i++){
				  ArrayList<HE_Vertex> vertexes = faces[i].vertices();
				  if(vertexes.size()>3){
					  //box.triangulateFace(faces[i].key(), true);
					  box.triSplitFace(faces[i]);
					  once_again = true;
				  }else{
					  blank.add(faces[i]);
				  }
			  }
			  once_count++;
		  }
		  
		  smooth();
		  
	}
	
	float count = 3f;
	float maxCount = 90f;

	public void draw() {
		background(255);
		
		int vertices_count = 0;
		HE_Face faces[] = box.facesAsArray();
		int cc = 0;
		for(int j=0; j<faces.length; j++){
			ArrayList<HE_Vertex> vertexes = faces[j].vertices();
			if(vertexes.size()!=3){
				println("oh no");
				cc++;
			}else{
				vertices_count += vertexes.size();
			}
		}
		
		float scaleFactor = 100f;
		
		float fvertices[] = new float[vertices_count*3];
		int ftriangles[] = new int[vertices_count];
		int fvertices_count = 0;
		int ftriangle_count = 0;
		for(int j=0; j<faces.length; j++){
			ArrayList<HE_Vertex> vertexes = faces[j].vertices();
			for(int i=0; i<vertexes.size(); i++){
				HE_Vertex vertexy = vertexes.get(i);
				fvertices[fvertices_count] = vertexy.xf()/scaleFactor;
				fvertices[fvertices_count+1] = vertexy.yf()/scaleFactor;
				fvertices[fvertices_count+2] = vertexy.zf()/scaleFactor;
				fvertices_count+=3;
			}
			  
			ftriangles[ftriangle_count] = ftriangle_count;
			ftriangles[ftriangle_count+1] = ftriangle_count+1;
			ftriangles[ftriangle_count+2] = ftriangle_count+2;
			ftriangle_count+=3;
		}
		  
		/*fill(255);
		stroke(0);
		for(int i=0; i<ftriangles.length; i+=3){
			beginShape();
			vertex(fvertices[ftriangles[i]*3], fvertices[ftriangles[i]*3+1], fvertices[ftriangles[i]*3+2]);
			vertex(fvertices[ftriangles[i+1]*3], fvertices[ftriangles[i+1]*3+1], fvertices[ftriangles[i+1]*3+2]);
			vertex(fvertices[ftriangles[i+2]*3], fvertices[ftriangles[i+2]*3+1], fvertices[ftriangles[i+2]*3+2]);
			endShape(CLOSE);
		}*/
		  
		  /*for(int i=0; i<fvertices.length; i+=12){
			  beginShape();
			  for(int j=0; j<12; j+=3){
				  vertex(fvertices[i+j], fvertices[i+j+1], fvertices[i+j+2]);
			  }
			  endShape(CLOSE);
		  }*/
			  
		  int samples = 32;
			  sunflow.setWidth(sceneWidth);
			sunflow.setHeight(sceneHeight);
			sunflow.setBackground(255, 255, 255);
			sunflow.setSunSkyLight("sunny");
			sunflow.setPointLight("myPointLight", new Point3(0f,5f,5f), new Color(255,255,255));
			sunflow.setDirectionalLight("myDirectionalLight", new Point3(-2f,3f,0f), new Vector3(0f,0f,0f), 3f, new Color(1f,0f,0f));

			/*camera(		0f, 0f, 10f, 
						(10f - (count/180f)*10f), (count/180f)*-10f, 0f, 
						0f, 1.0f, 0f);*/
			
			camera(	0f, 0f, 7f-(5f/maxCount)*count, 
					5f + count*(14f/maxCount), (-20f/maxCount)*count, 0f, 
					0f, 1.0f, 0f);
			
			/*camera(	0f, 0f, 5f, 
					10f, -20f, 0f, 
					0f, 1.0f, 0f);*/
			
			sunflow.setThinlensCamera("thinLensCamera", 50f, (float)sceneWidth/sceneHeight);
			sunflow.setAmbientOcclusionShader(new Color(255,255,255), new Color(0,0,0), samples, 1f);
			sunflow.drawPlane("ground", new Point3(0f,-1.4f,0f), new Vector3(0f,1f,0f));
			sunflow.setAmbientOcclusionShader(new Color(255,255,255), new Color(255,255,255), samples, 1f);
			sunflow.drawPlane("back", new Point3(0,0,-100f), new Vector3(0,0,1));
			
			sunflow.setShinyDiffuseShader(new Color(10,10,10), 10f);
			sunflow.drawMesh("hemesh", fvertices, ftriangles);
			//sunflow.rotate(1f);
			//sunflow.drawSphereFlake("mySphereFlake", 2, new Vector3(0,1f,0), 1);
			
			sunflow.setPathTracingGIEngine(samples);
			sunflow.render("SunflowTestRender_box_"+round(count)+".png"); //  //year()+"-"+month()+"-"+day()+"_"+hour()+"-"+minute()+"-"+second()+".png"); 
			count++;
			
			if(count>maxCount){
				exit();
			}
	}
	
	public static void main(String _args[]) {
		PApplet.main(new String[] { hemesher.Hemesher.class.getName() });
	}
}
